@@ -17,7 +17,7 @@
 
 #include <stdio.h>
 
-#include <cmath>  // std::ceil, std::floor
+#include <cmath>  // std::ceil, std::floor, std::log2
 
 #include "hwy/base.h"
 
@@ -30,6 +30,7 @@
 HWY_BEFORE_NAMESPACE();
 namespace hwy {
 namespace HWY_NAMESPACE {
+namespace {
 
 HWY_NOINLINE void TestAllF16FromF32() {
   const FixedTag<float, 1> d1;
@@ -143,8 +144,9 @@ struct TestApproximateReciprocal {
     double worst_expected = 0.0;
     double worst_actual = 0.0;
     for (size_t i = 0; i < N; ++i) {
-      const double expected = 1.0 / input[i];
-      const double l1 = ScalarAbs(expected - actual[i]);
+      const double expected = 1.0 / ConvertScalarTo<double>(input[i]);
+      const double l1 =
+          ScalarAbs(expected - ConvertScalarTo<double>(actual[i]));
       if (l1 > max_l1) {
         max_l1 = l1;
         worst_expected = expected;
@@ -165,6 +167,53 @@ HWY_NOINLINE void TestAllApproximateReciprocal() {
   ForFloatTypes(ForPartialVectors<TestApproximateReciprocal>());
 }
 
+struct TestMaskedApproximateReciprocal {
+  template <typename T, class D>
+  HWY_NOINLINE void operator()(T /*unused*/, D d) {
+    const MFromD<D> first_three = FirstN(d, 3);
+    const auto v = Iota(d, -2);
+    const auto nonzero =
+        IfThenElse(Eq(v, Zero(d)), Set(d, ConvertScalarTo<T>(1)), v);
+    const size_t N = Lanes(d);
+    auto input = AllocateAligned<T>(N);
+    auto actual = AllocateAligned<T>(N);
+    HWY_ASSERT(input && actual);
+
+    Store(nonzero, d, input.get());
+    Store(MaskedApproximateReciprocal(first_three, nonzero), d, actual.get());
+
+    double max_l1 = 0.0;
+    double worst_expected = 0.0;
+    double worst_actual = 0.0;
+    double expected;
+    for (size_t i = 0; i < N; ++i) {
+      if (i < 3) {
+        expected = 1.0 / ConvertScalarTo<double>(input[i]);
+      } else {
+        expected = 0.0;
+      }
+      const double l1 =
+          ScalarAbs(expected - ConvertScalarTo<double>(actual[i]));
+      if (l1 > max_l1) {
+        max_l1 = l1;
+        worst_expected = expected;
+        worst_actual = actual[i];
+      }
+    }
+    const double abs_worst_expected = ScalarAbs(worst_expected);
+    if (abs_worst_expected > 1E-5) {
+      const double max_rel = max_l1 / abs_worst_expected;
+      fprintf(stderr, "max l1 %f rel %f (%f vs %f)\n", max_l1, max_rel,
+              worst_expected, worst_actual);
+      HWY_ASSERT(max_rel < 0.004);
+    }
+  }
+};
+
+HWY_NOINLINE void TestAllMaskedApproximateReciprocal() {
+  ForFloatTypes(ForPartialVectors<TestMaskedApproximateReciprocal>());
+}
+
 struct TestSquareRoot {
   template <typename T, class D>
   HWY_NOINLINE void operator()(T /*unused*/, D d) {
@@ -177,6 +226,26 @@ HWY_NOINLINE void TestAllSquareRoot() {
   ForFloatTypes(ForPartialVectors<TestSquareRoot>());
 }
 
+struct TestMaskedSqrt {
+  template <typename T, class D>
+  HWY_NOINLINE void operator()(T /*unused*/, D d) {
+    const auto v0 = Zero(d);
+    const auto vi = Iota(d, 4);
+    const auto v2 = Iota(d, 5);
+
+    const MFromD<D> first_four = FirstN(d, 4);
+    const auto expected = IfThenElse(first_four, Sqrt(vi), v0);
+    const auto masked_expected = IfThenElse(first_four, Sqrt(vi), v2);
+
+    HWY_ASSERT_VEC_EQ(d, expected, MaskedSqrt(first_four, vi));
+    HWY_ASSERT_VEC_EQ(d, masked_expected, MaskedSqrtOr(v2, first_four, vi));
+  }
+};
+
+HWY_NOINLINE void TestAllMaskedSqrt() {
+  ForFloatTypes(ForPartialVectors<TestMaskedSqrt>());
+}
+
 struct TestReciprocalSquareRoot {
   template <typename T, class D>
   HWY_NOINLINE void operator()(T /*unused*/, D d) {
@@ -187,7 +256,9 @@ struct TestReciprocalSquareRoot {
     Store(ApproximateReciprocalSqrt(v), d, lanes.get());
     for (size_t i = 0; i < N; ++i) {
       T err = ConvertScalarTo<T>(ConvertScalarTo<float>(lanes[i]) - 0.090166f);
-      if (err < ConvertScalarTo<T>(0)) err = -err;
+      if (err < ConvertScalarTo<T>(0)) {
+        err = ConvertScalarTo<T>(-ConvertScalarTo<float>(err));
+      }
       if (static_cast<double>(err) >= 4E-4) {
         HWY_ABORT("Lane %d (%d): actual %f err %f\n", static_cast<int>(i),
                   static_cast<int>(N), static_cast<double>(lanes[i]),
@@ -201,6 +272,35 @@ HWY_NOINLINE void TestAllReciprocalSquareRoot() {
   ForFloatTypes(ForPartialVectors<TestReciprocalSquareRoot>());
 }
 
+struct TestMaskedReciprocalSquareRoot {
+  template <typename T, class D>
+  HWY_NOINLINE void operator()(T /*unused*/, D d) {
+    const Vec<D> v = Set(d, ConvertScalarTo<T>(123.0f));
+    const MFromD<D> first_three = FirstN(d, 3);
+    const size_t N = Lanes(d);
+    auto lanes = AllocateAligned<T>(N);
+    HWY_ASSERT(lanes);
+    Store(MaskedApproximateReciprocalSqrt(first_three, v), d,
+          lanes.get());
+    for (size_t i = 0; i < N; ++i) {
+      T expected_val = i < 3 ? ConvertScalarTo<T>(1 / std::sqrt(123.0f))
+                             : ConvertScalarTo<T>(0);
+      T err =
+          ConvertScalarTo<T>(ConvertScalarTo<float>(lanes[i]) - expected_val);
+      if (err < ConvertScalarTo<T>(0)) err = -err;
+      if (static_cast<double>(err) >= 4E-4) {
+        HWY_ABORT("Lane %d (%d): actual %f err %f\n", static_cast<int>(i),
+                  static_cast<int>(N), static_cast<double>(lanes[i]),
+                  static_cast<double>(err));
+      }
+    }
+  }
+};
+
+HWY_NOINLINE void TestAllMaskedReciprocalSquareRoot() {
+  ForFloatTypes(ForPartialVectors<TestMaskedReciprocalSquareRoot>());
+}
+
 template <typename T, class D>
 AlignedFreeUniquePtr<T[]> RoundTestCases(T /*unused*/, D d, size_t& padded) {
   const T eps = Epsilon<T>();
@@ -505,21 +605,60 @@ HWY_NOINLINE void TestAllAbsDiff() {
   ForFloatTypes(ForPartialVectors<TestAbsDiff>());
 }
 
+// Test GetExponent
+struct TestGetExponent {
+  template <typename T, class D>
+  HWY_NOINLINE void operator()(T /*unused*/, D d) {
+    const RebindToUnsigned<decltype(d)> du;
+
+    using TFArith = If<IsSpecialFloat<T>(), float, T>;
+    using TU = MakeUnsigned<T>;
+    const size_t N = Lanes(d);
+
+    auto v = Iota(d, 1);
+
+    auto expected = AllocateAligned<T>(N);
+    auto expected_biased = AllocateAligned<TU>(N);
+    HWY_ASSERT(expected && expected_biased);
+
+    constexpr int kNumOfMantBits = MantissaBits<T>();
+
+    for (size_t i = 0; i < N; ++i) {
+      const T test_val = ConvertScalarTo<T>(i + 1);
+      expected[i] = ConvertScalarTo<T>(
+          std::floor(std::log2(ConvertScalarTo<TFArith>(test_val))));
+      expected_biased[i] =
+          static_cast<TU>((BitCastScalar<TU>(test_val) >> kNumOfMantBits) &
+                          static_cast<TU>(MaxExponentField<T>()));
+    }
+    HWY_ASSERT_VEC_EQ(d, expected.get(), GetExponent(v));
+    HWY_ASSERT_VEC_EQ(du, expected_biased.get(), GetBiasedExponent(v));
+  }
+};
+
+HWY_NOINLINE void TestAllGetExponent() {
+  ForFloatTypes(ForPartialVectors<TestGetExponent>());
+}
+
+}  // namespace
 // NOLINTNEXTLINE(google-readability-namespace-comments)
 }  // namespace HWY_NAMESPACE
 }  // namespace hwy
 HWY_AFTER_NAMESPACE();
 
 #if HWY_ONCE
-
 namespace hwy {
+namespace {
 HWY_BEFORE_TEST(HwyFloatTest);
 HWY_EXPORT_AND_TEST_P(HwyFloatTest, TestAllF16FromF32);
 HWY_EXPORT_AND_TEST_P(HwyFloatTest, TestAllF32FromF16);
 HWY_EXPORT_AND_TEST_P(HwyFloatTest, TestAllDiv);
 HWY_EXPORT_AND_TEST_P(HwyFloatTest, TestAllApproximateReciprocal);
 HWY_EXPORT_AND_TEST_P(HwyFloatTest, TestAllSquareRoot);
+HWY_EXPORT_AND_TEST_P(HwyFloatTest, TestAllMaskedSqrt);
 HWY_EXPORT_AND_TEST_P(HwyFloatTest, TestAllReciprocalSquareRoot);
+HWY_EXPORT_AND_TEST_P(HwyFloatTest, TestAllMaskedReciprocalSquareRoot);
+HWY_EXPORT_AND_TEST_P(HwyFloatTest, TestAllMaskedApproximateReciprocal);
 HWY_EXPORT_AND_TEST_P(HwyFloatTest, TestAllRound);
 HWY_EXPORT_AND_TEST_P(HwyFloatTest, TestAllNearestInt);
 HWY_EXPORT_AND_TEST_P(HwyFloatTest, TestAllDemoteToNearestInt);
@@ -527,7 +666,9 @@ HWY_EXPORT_AND_TEST_P(HwyFloatTest, TestAllTrunc);
 HWY_EXPORT_AND_TEST_P(HwyFloatTest, TestAllCeil);
 HWY_EXPORT_AND_TEST_P(HwyFloatTest, TestAllFloor);
 HWY_EXPORT_AND_TEST_P(HwyFloatTest, TestAllAbsDiff);
+HWY_EXPORT_AND_TEST_P(HwyFloatTest, TestAllGetExponent);
 HWY_AFTER_TEST();
+}  // namespace
 }  // namespace hwy
-
-#endif
+HWY_TEST_MAIN();
+#endif  // HWY_ONCE
