@@ -45,6 +45,12 @@ bool is_a_nan(T elem)
     return std::isnan(elem);
 }
 
+template <>
+X86_SIMD_SORT_INLINE_ONLY bool is_a_nan<uint16_t>(uint16_t elem)
+{
+    return ((elem & 0x7c00u) == 0x7c00u) && ((elem & 0x03ffu) != 0);
+}
+
 template <typename vtype, typename T>
 X86_SIMD_SORT_INLINE arrsize_t replace_nan_with_inf(T *arr, arrsize_t size)
 {
@@ -110,7 +116,7 @@ X86_SIMD_SORT_INLINE void replace_inf_with_nan(type_t *arr,
                 arr[ii] = xss::fp::quiet_NaN<type_t>();
             }
             else {
-                arr[ii] = 0xFFFF;
+                arr[ii] = 0x7c01; // std::quiet_nan
             }
             nan_count -= 1;
         }
@@ -121,7 +127,7 @@ X86_SIMD_SORT_INLINE void replace_inf_with_nan(type_t *arr,
                 arr[ii] = xss::fp::quiet_NaN<type_t>();
             }
             else {
-                arr[ii] = 0xFFFF;
+                arr[ii] = 0x7c01; // std::quiet_nan
             }
             nan_count -= 1;
         }
@@ -666,9 +672,7 @@ X86_SIMD_SORT_INLINE void xss_qsort(T *arr, arrsize_t arrsize, bool hasnan)
         bool use_parallel = arrsize > 100000;
 
         if (use_parallel) {
-            // This thread limit was determined experimentally; it may be better for it to be the number of physical cores on the system
-            constexpr int thread_limit = 8;
-            int thread_count = std::min(thread_limit, omp_get_max_threads());
+            int thread_count = xss_get_num_threads();
             arrsize_t task_threshold
                     = std::max((arrsize_t)100000, arrsize / 100);
 
